package com.hzya.frame.plugin.lets.plugin.base;

import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.entity.BdInvbasdocEntity;
import com.hzya.frame.plugin.lets.queryvo.CustDocAssginVo;
import com.hzya.frame.plugin.lets.queryvo.CustdocRequestVo;
import com.hzya.frame.plugin.lets.resultvo.CbdocResultVo;
import com.hzya.frame.plugin.lets.u8cdto.ReusltStrDto;
import com.hzya.frame.plugin.lets.util.PushDataByU8cUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @Author
 * @Date 2024/8/12 13:57
 * @PackageName:com.hzya.frame.plugin.lets.plugin.base
 * @ClassName: CustdocAutoAllotPluginInitializer
 * @Description: 客商档案自动分配
 */
public class CustdocAutoAllotPluginInitializer extends PluginBaseEntity {

    Logger logger = LoggerFactory.getLogger(GoodsAutoAllotPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "com.hzya.frame.plugin.lets.plugin.base.GoodsAutoAllotPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：U8C->客商基本档案自动分配全公司";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：U8C->客商基本档案自动分配全公司";
    }

    @Override
    public String getPluginType() {
        return "1";
    }

    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;
    @Autowired
    private PushDataByU8cUtil pushDataByU8cUtil;
    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        return null;
    }


    /**
     * 默认推送
     */
    public void start() {
        try {
            //查询要分配的客商
            List<CustdocRequestVo> list = queryCustDoc();
            //推送：自动分配
            implement(list);

        } catch (Exception e) {
            logger.error("丽知：U8C->客商基本档案自动分配全公司,start()方法报错：", e);
        }
    }



    /**
     * 过滤成功日志
     *
     * @return
     */
    public List<BdInvbasdocEntity> filter(List<BdInvbasdocEntity> bdInvbasdocEntityList) {
        List<BdInvbasdocEntity> filterInvbasdocList = new ArrayList<>();
        for (BdInvbasdocEntity bdInvbasdocEntity : bdInvbasdocEntityList) {
            String rootAppPk = bdInvbasdocEntity.getPkInvbasdoc();
            boolean isExis = true;
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setRootAppPk(rootAppPk);
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = iIntegrationTaskLivingDetailsDao.query(integrationTaskLivingDetailsEntity);
            if (integrationTaskLivingDetailsEntities == null || integrationTaskLivingDetailsEntities.size() == 0) {
                isExis = false;
            }
            if (!isExis) {
                filterInvbasdocList.add(bdInvbasdocEntity);
            }

        }
        return filterInvbasdocList;
    }

    /**
     * 查询客商基本档案
     * @return
     */
    private List<CustdocRequestVo> queryCustDoc() {
        List<CustdocRequestVo> custdocRequestVos=null;
        try {
            String data = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocQuery"), "{}");
            CbdocResultVo cbdocResultVo = JSONObject.parseObject(data, CbdocResultVo.class);
            String datastring = cbdocResultVo.getData();
            List list = JSONObject.parseObject(datastring, List.class);
            custdocRequestVos = filterData(list);
        } catch (Exception e) {
            logger.error("丽知：U8C->客商基本档案自动分配全公司,查询客商基本档案失败", e);
        }
        return custdocRequestVos;
    }

    private List<CustdocRequestVo> filterData(List list) {
        List<CustdocRequestVo> filterlist = new ArrayList<>();
        for (Object o : list) {
           //o转为
            CustdocRequestVo requestVo = JSONObject.parseObject(JSONObject.toJSONString(o), CustdocRequestVo.class);


            if("N".equals(requestVo.getParentvo().getDef10())){
                filterlist.add(requestVo);
            }
        }
        return filterlist;
    }

    /**
     * U8C->客商基本档案自动分配全公司
     */
    public void implement(List<CustdocRequestVo> list) {

        //查询公司档案
        try {
            String corpresponse = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("bdcorpQuery"), "{}");
            CbdocResultVo cbdocResultVo = JSONObject.parseObject(corpresponse, CbdocResultVo.class);
            if(cbdocResultVo.getStatus()!="success"){
                Assert.state(false, "查询公司档案失败");
            }

        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        for (CustdocRequestVo custdocRequestVo : list) {
            String assginjsonString = "";
            String pkCubasdoc = custdocRequestVo.getParentvo().getPk_cubasdoc();
            String createtime = custdocRequestVo.getParentvo().getCreatetime();
            try {
                ArrayList<CustDocAssginVo> assginvolist = new ArrayList<>();
                CustDocAssginVo custDocAssginVo = new CustDocAssginVo();
                custDocAssginVo.setPk_cubasdoc(pkCubasdoc);
                custDocAssginVo.setAssignallcorps("Y");
                custDocAssginVo.setCustprop("2");
                assginvolist.add(custDocAssginVo);

                HashMap<String, List> map = new HashMap<>();
                map.put("custbasvo", list);
                assginjsonString = JSONObject.toJSONString(map);
                String response = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocAssign"), assginjsonString);

                System.out.println(response);
                boolean isSuccess = false;
                if (response != null && !"".equals(response)) {
                    ReusltStrDto reusltStrDto = JSON.parseObject(response, ReusltStrDto.class);
                    if ("success".equals(reusltStrDto.getStatus())) {
                        isSuccess = true;
                        //分配成功去更新状态
                        updateDoc(custdocRequestVo);
                    }
                }
                if (!isSuccess) {
                    Assert.state(false, "推送U8C--> U8C->客商基本档案自动分配全公司 失败 接口返回结果：{} 接口入参:{}", response, assginjsonString);
                }
                //成功
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                integrationTaskLivingDetailsEntity.setRootAppNewData(assginjsonString);
                integrationTaskLivingDetailsEntity.setNewTransmitInfo(response);
                integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                integrationTaskLivingDetailsEntity.setBusinessDate(createtime);
                integrationTaskLivingDetailsEntity.setRootAppPk(pkCubasdoc);
                integrationTaskLivingDetailsEntity.setRootAppBill(pkCubasdoc);
                integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
            } catch (Exception e) {
                e.printStackTrace();
                logger.error("U8C->客商基本档案自动分配全公司implement(List<BdInvbasdocEntity> filterInvbasdocList)方法失败");
                //失败
                String ErrMessage = e.getMessage();
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                integrationTaskLivingDetailsEntity.setRootAppNewData(assginjsonString);
                integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
                integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                integrationTaskLivingDetailsEntity.setBusinessDate(createtime);
                integrationTaskLivingDetailsEntity.setRootAppPk(pkCubasdoc);
                integrationTaskLivingDetailsEntity.setRootAppBill(pkCubasdoc);
                integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
            }
        }
    }

    private void updateDoc(CustdocRequestVo custdocRequestVo) {
        //已分配
        String pkCubasdoc = custdocRequestVo.getParentvo().getPk_cubasdoc();
        String createtime = custdocRequestVo.getParentvo().getCreatetime();

        custdocRequestVo.getParentvo().setDef10("Y");
        ArrayList<CustdocRequestVo> requestVos = new ArrayList<>();
        requestVos.add(custdocRequestVo);
        HashMap<String, List> map = new HashMap<>();
        map.put("custbasvo", requestVos);
        String data = JSONObject.toJSONString(map);
        try {
            String response = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocUpdate"), data);

            boolean isSuccess = false;
            if (response != null && !"".equals(response)) {
                ReusltStrDto reusltStrDto = JSON.parseObject(response, ReusltStrDto.class);
                if ("success".equals(reusltStrDto.getStatus())) {
                    isSuccess = true;
                }
            }
            if (!isSuccess) {
                Assert.state(false, "推送U8C--> U8C->客商基本档案自动分配全公司 失败 接口返回结果：{} 接口入参:{}", response, data);
            }
            //成功
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
            integrationTaskLivingDetailsEntity.setRootAppNewData(data);
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(response);
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(createtime);
            integrationTaskLivingDetailsEntity.setRootAppPk(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setRootAppBill(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);

        } catch (Exception e) {
            //失败
            logger.error("U8C->客商基本档案更新updateDoc(CustdocRequestVo custdocRequestVo)方法失败");
            String ErrMessage = e.getMessage();
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData(data);
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(createtime);
            integrationTaskLivingDetailsEntity.setRootAppPk(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setRootAppBill(pkCubasdoc);
            integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
        }

    }
}

