package com.hzya.frame.plugin.lets.plugin.base;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.entity.BdCubasdocEntity;
import com.hzya.frame.plugin.lets.ofs.entity.ShopEntity;
import com.hzya.frame.plugin.lets.ofs.entity.SupplierEntity;

import com.hzya.frame.plugin.lets.queryvo.CustDocAssginVo;
import com.hzya.frame.plugin.lets.queryvo.CustdocRequestVo;
import com.hzya.frame.plugin.lets.queryvo.CustmandocRequestVo;
import com.hzya.frame.plugin.lets.resultvo.CbdocResultVo;
import com.hzya.frame.plugin.lets.util.PushDataByU8cUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @Author
 * @Date 2024/8/6 14:36
 * @PackageName:com.hzya.frame.plugin.lets.plugin.base
 * @ClassName: ShopPluginInitializer
 * @Description: 供应商档案->客商档案
 */
public class SupplierPluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(GoodsClassPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "执行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "执行销毁方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "ShopPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：OFS供应商档案--->U8C客商档案";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：OFS供应商档案--->U8C客商档案";
    }

    @Override
    public String getPluginType() {
        return "1";
    }


    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;


    @Autowired
    private PushDataByU8cUtil pushDataByU8cUtil;

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        String param = String.valueOf(requestJson.get("param"));
        if (requestJson != null && ProfilesActiveConstant.TYPE_VBILLCODE.equals(requestJson.get("type"))) {
            if (param != null && !"".equals(param)) {
                startImplement(param);
            }
        } else if (ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
            String integration_task_living_details_id = (String) requestJson.get("integration_task_living_details_id");
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = saveOrUpdateBusinessLogUtil.queryIntegrationTaskLivingDetailsEntity(integration_task_living_details_id);
            startImplement(integrationTaskLivingDetailsEntity.getRootAppPk());
        } else {
            startImplement();
        }
        return null;
    }


    /**
     * 默认推送
     */
    public void startImplement() {
        try {
            //获取当前时间
            //获取当前时间
            Date currentDate = new Date();
            Date startTime = DateUtil.offset(currentDate, DateField.MINUTE, -10);
            Date endTime = DateUtil.offset(currentDate, DateField.MINUTE, -2);
            String startTimeStr = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
            String endTimeStr = DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss");

            SupplierEntity supplierEntity = new SupplierEntity();

            //TODO ofs查询店铺信息
//            List<ShopEntity> query = shopDao.query(shopEntity);
            List<SupplierEntity> query = null;
            //过滤数据
            List<SupplierEntity> supplierEntities = filterSupplierEntity(query);

            implement(supplierEntities);

        } catch (Exception e) {
            logger.error("丽知：OFS供应商档案--->U8C客商档案,startImplement()方法报错：", e);
        }
    }


    /**
     * 按编码名称
     */
    public void startImplement(String code) {
        Assert.notNull(code, "code不能为空");
        try {
            ShopEntity shopEntity = new ShopEntity();
            shopEntity.setBizCode(code);
            //TODO ofs查询供应商信息

            List<SupplierEntity> query = null;
            //过滤数据
            List<SupplierEntity> supplierEntities = filterSupplierEntity(query);
            //执行
            implement(supplierEntities);

        } catch (Exception e) {
            logger.error("丽知：OFS供应商档案--->U8C客商档案,startImplement(String code)方法报错：", e);
        }
    }

    /**
     * 按时间区间
     */
    public void startImplement(String startTime, String endTime) {
        try {
            Date business_start = DateUtil.parse(startTime);
            Date business_end = DateUtil.parse(endTime);
            String start = DateUtil.format(business_start, "yyyy-MM-dd") + " 00:00:00";
            String end = DateUtil.format(business_end, "yyyy-MM-dd") + " 23:59:59";

            ShopEntity shopEntity = new ShopEntity();
            shopEntity.setCreated_start(start);
            shopEntity.setCreated_end(end);
            //TODO ofs查询供应商信息
        } catch (Exception e) {
            logger.error("丽知：OFS供应商档案--->U8C客商档案,startImplement(String startTime, String endTime)方法报错：", e);
        }
    }

    public void implement(List<SupplierEntity> shopEntityList) {

        Assert.notNull(shopEntityList, "shopEntityList不能为空");
        //业务时间
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = new Date(System.currentTimeMillis());
        String businessFormat = sdf.format(date);
        for (SupplierEntity supplierEntity : shopEntityList) {
            BdCubasdocEntity parentvo = new BdCubasdocEntity();
            CustdocRequestVo BdCubasdocEntity = new CustdocRequestVo();
            try {
                //TODO 设置parentvo
//                parentvo.setPk_areacl("");
                parentvo.setCustname(supplierEntity.getName());
                parentvo.setCustshortname(supplierEntity.getName());
                parentvo.setCustcode(supplierEntity.getCompanyCode());

                BdCubasdocEntity.setParentvo(parentvo);
                //TODO 同步u8c
                HashMap<String, Object> param = new HashMap<>();
                param.put("cbdocvo", BdCubasdocEntity);

                String jsonString = JSONObject.toJSONString(param);
                String response = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocSave"), jsonString);
                if (response != null) {
                    CbdocResultVo cbdocResultVo = JSONObject.parseObject(response, CbdocResultVo.class);
                    String taskNumber = null;
                    boolean isSuccess = false;
                    if ("success".equals(cbdocResultVo.getStatus())) {
                        isSuccess = true;
                        taskNumber = cbdocResultVo.getTaskNumber();
                        String data = cbdocResultVo.getData();

//                        CustdocRequestVo BdCubasdocEntity1 = JSONObject.parseObject(data, CustdocRequestVo.class);
                        List datalist = JSONObject.parseObject(data, List.class);
                        Object o = datalist.get(0);
                        CustdocRequestVo BdCubasdocEntity1 = JSONObject.parseObject(JSONObject.toJSONString(o), CustdocRequestVo.class);


                        CustDocAssginVo custDocAssginVo = new CustDocAssginVo();
                        custDocAssginVo.setAssignallcorps("Y");
                        //分配给所有公司
                        custDocAssginVo.setCustprop("2");
                        custDocAssginVo.setPk_cubasdoc(BdCubasdocEntity1.getParentvo().getPk_cubasdoc());
                        HashMap<String, Object> assginHashMap = new HashMap<>();
                        assginHashMap.put("custbasvo", custDocAssginVo);
                        String assginjsonString = JSONObject.toJSONString(assginHashMap);
                        String assignresponse = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custdocAssgin"), assginjsonString);
                        CbdocResultVo assginCbdocResultVo = JSONObject.parseObject(assignresponse, CbdocResultVo.class);
                        if (!"success".equals(assginCbdocResultVo.getStatus())) {
                            isSuccess = false;
                        }

                        //生成客商管理档案

                        //地区名称：丽知商城客商？
                        parentvo.setPk_areacl("F");
                        parentvo.setCustprop(BdCubasdocEntity1.getParentvo().getCustprop());
                        CustmandocRequestVo custmandocRequestVo = new CustmandocRequestVo();
                        custmandocRequestVo.setParentvo(parentvo);
//                        BdCumandocEntity BdCumandocEntity = new BdCumandocEntity();
//                        BdCumandocEntity.setCustflag("1");
//
//                        BdCumandocEntity BdCumandocEntity1 = new BdCumandocEntity();
//                        BdCumandocEntity.setCustflag(" ");
//                        ArrayList<BdCumandocEntity> list = new ArrayList<>();
//                        list.add(BdCumandocEntity);
//                        list.add(BdCumandocEntity1);
//                        custmandocRequestVo.setChildrenvo(list);

                        //TODO 设置属性
                        HashMap<String, Object> mandocparam = new HashMap<>();
                        mandocparam.put("billvo", custmandocRequestVo);
                        String mandocjsonString = JSONObject.toJSONString(mandocparam);
                        String mandocresponse = pushDataByU8cUtil.pushU8CByCode(OverallConstant.getOverAllValue("custmandocSave"), mandocjsonString);
                        CbdocResultVo mandoccbdocResultVo = JSONObject.parseObject(mandocresponse, CbdocResultVo.class);
                        if (!"success".equals(mandoccbdocResultVo.getStatus())) {
                            isSuccess = false;
                        }
                    }
                    if (!isSuccess) {
                        Assert.state(false, "推送U8C，原因：{} 接口入参:{} {}", JSON.toJSONString(cbdocResultVo), JSON.toJSONString(OverallConstant.getOverAllValue("custdocSave")), JSON.toJSONString(BdCubasdocEntity));
                    }
                    //TODO 根据返回结果记录日志
                    //成功
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                    integrationTaskLivingDetailsEntity.setRootAppNewData(jsonString);
                    integrationTaskLivingDetailsEntity.setNewTransmitInfo(JSON.toJSONString(cbdocResultVo));
                    integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                    integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
                    integrationTaskLivingDetailsEntity.setRootAppPk(supplierEntity.getBizCode());
                    integrationTaskLivingDetailsEntity.setRootAppBill(supplierEntity.getName());
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                    //TODO 更新供应商状态
                }


            } catch (Exception e) {
                e.printStackTrace();
                String ErrMessage = e.getMessage();
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                integrationTaskLivingDetailsEntity.setRootAppNewData(JSONObject.toJSONString(BdCubasdocEntity));
                integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
                integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                integrationTaskLivingDetailsEntity.setBusinessDate(businessFormat);
                integrationTaskLivingDetailsEntity.setRootAppPk(supplierEntity.getBizCode());
                integrationTaskLivingDetailsEntity.setRootAppBill(supplierEntity.getName());
                integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                logger.error("丽知：OFS供应商档案--->U8C客商档案,implement()方法报错：", e);
            }
        }
    }

    //过滤数据
    public List<SupplierEntity> filterSupplierEntity(List<SupplierEntity> list) {

        Assert.notNull(list, "不能为空！");

        List<SupplierEntity> supplierEntityList = new ArrayList<>();
        if (list != null && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                SupplierEntity supplierEntity = list.get(i);

                if (supplierEntity.getBizCode() != null && !"".equals(supplierEntity.getBizCode().trim())) {
                    boolean isExis = true;
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setRootAppPk(supplierEntity.getBizCode());
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y_H);
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = iIntegrationTaskLivingDetailsDao.query(integrationTaskLivingDetailsEntity);
                    if (integrationTaskLivingDetailsEntities == null || integrationTaskLivingDetailsEntities.size() == 0) {
                        isExis = false;
                    }
                    if (!isExis) {
                        supplierEntityList.add(supplierEntity);
                    }
                }
            }
        }
        return supplierEntityList;
    }


}
